<?php

/**
 * Copyright © 2019-present Simplr Solutions Inc. All rights reserved
 */

namespace Simplr\SimplrExtension\Model;

use Magento\Framework\App\State;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Sales\Api\Data\OrderInterface as MagentoOrder;
use Magento\Sales\Api\Data\OrderSearchResultInterface as OrderSearchResult;
use Magento\Framework\Api\SearchCriteriaInterface as SearchCriteria;
use Magento\Sales\Api\Data\CreditmemoCommentCreationInterface as CreditmemoCommentCreation;
use Magento\Sales\Api\Data\CreditmemoCreationArgumentsInterface as CreditmemoCreationArgument;
use Magento\Sales\Api\Data\CreditmemoItemCreationInterface as CreditmemoItemCreation;
use Magento\Sales\Api\RefundOrderInterface;
use Magento\Sales\Model\Order;

class SimplrAPI implements \Simplr\SimplrExtension\Api\SimplrAPIInterface
{
    /**
     * @var State
     */
    protected $state;

    /**
     * @var ModuleListInterface
     */

    protected $moduleList;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var RefundOrderInterface
     */
    private $refundOrder;

    /**
     * SimplrAPI constructor.
     *
     * @param State $state
     * @param OrderRepositoryInterface $orderRepository
     * @param ModuleListInterface $moduleList
     * @param RefundOrderInterface $refundOrder
     */
    public function __construct(
        State $state,
        OrderRepositoryInterface $orderRepository,
        ModuleListInterface $moduleList,
        RefundOrderInterface $refundOrder
    ) {
        $this->state = $state;
        $this->moduleList = $moduleList;
        $this->refundOrder = $refundOrder;
        $this->orderRepository = $orderRepository;
    }

    /**
     * Returns extension version
     *
     * @return string
     */
    public function versionAPI()
    {
        $moduleCode = 'Simplr_SimplrExtension';
        $moduleInfo = $this->moduleList->getOne($moduleCode);
        return $moduleInfo['setup_version'];
    }

     /**
      * Returns list of orders by search criteria
      *
      * About search criteria - https://devdocs.magento.com/guides/v2.3/rest/performing-searches.html
      *
      * @param SearchCriteria $searchCriteria The search criteria.
      * @return OrderSearchResult Order search result interface.
      */
    public function getOrdersAPI(SearchCriteria $searchCriteria)
    {
        return $this->orderRepository->getList($searchCriteria);
    }

    /**
     * Returns a order by id.
     *
     * @param int $orderId The order ID.
     * @return MagentoOrder Order interface.
     */
    public function getOrderAPI($orderId)
    {
        return $this->getOrder($orderId);
    }

    /**
     * Cancel order by id
     *
     * @param int $orderId The order ID.
     * @return MagentoOrder
     */
    public function cancelOrderAPI($orderId)
    {
        $order = $this->getOrder($orderId);
        if (!$order->canCancel()) {
            throw new \Magento\Framework\Webapi\Exception(
                __('Order cann\'t cancel.'),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_BAD_REQUEST
            );
        }

        $order->addStatusHistoryComment('Order has been canceled by Simplr', Order::STATE_CANCELED);
        $order->cancel();
        $order->save();
        return $this->getOrder($orderId);
    }

    /**
     * Offline refund items of the order
     *
     * @param MagentoOrder $order
     * @param CreditmemoItemCreation[] $items
     * @param bool $notify
     * @param bool $appendComment
     * @param CreditmemoCommentCreation|null $comment
     * @param CreditmemoCreationArgument|null $arguments
     * @return MagentoOrder
     */
    public function refundOrderAPI(
        $orderId,
        array $items = [],
        $notify = false,
        $appendComment = false,
        CreditmemoCommentCreation $comment = null,
        CreditmemoCreationArgument $arguments = null
    ) {
        $this->refundOrder->execute($orderId, $items, $notify, $appendComment, $comment, $arguments);
        return $this->getOrder($orderId);
    }

    /**
     * Protected - Returns a scecific order
     *
     * @param int $orderId The order ID.
     * @return MagentoOrder
     */
    protected function getOrder($orderId)
    {
        return $this->orderRepository->get($orderId);
    }

    /**
     * Protected - Returns status of develop mode
     *
     * @return bool
     */
    protected function _isDeveloperMode()
    {
        return $this->state->getMode() == State::MODE_DEVELOPER;
    }
}
